#################################################################################
## AQS DATA PROCESSING
## 
## Developed by: US EPA, Office of Air Quality Planning and Standards
## August 2019
## 
## This script Reshapes monitor data from AQS (https://aqs.epa.gov/aqsweb/airdata/download_files.html#Raw)
## into BenMAP-ready input files.
#################################################################################

# Install necessary libraries
if (!require(padr)) install.packages("padr")
library(padr)

if (!require(data.table)) install.packages("data.table")
library(data.table)

##
## !!USER UPDATES REQUIRED!! Update file directory
## 

# Specify directory containing AQS downloads
# Sample path (note use of "/" instead of "\"): C:/Users/yourname/Desktop/Temp
path<- "J:/tmp/downloads"
setwd(path)

# Create ouput directory for BenMAP-ready files; if the folder already exists 
# a warning message will appear and can be ignored
dir.create(paste(path,"/BenMAP Files",sep=""))

##
## Format all AQS downloads
##


# Creates a list of all zipped files in the AQS downloads folder
zips<- dir(path, pattern =".zip")

# Unzips all zipped files in the AQS downloads folders
for(i in 1:length(zips)){
  unzip(zips[i])
}

# Creates a list of all .csv files in the AQS downloads folder
files<- dir(path, pattern =".csv")

# Define relevant columns for both PM and ozone datasets
dataCol<- c(NA, NA, NA, NA, NA, NA, NA, "NULL", "NULL", NA, "NULL", NA, "NULL", NA,"NULL",NA,NA,NA,"NULL","NULL",NA,"NULL","NULL","NULL","NULL","NULL","NULL","NULL","NULL")

# Loop over each AQS download
for(i in 1:length(files)){
  
  # Set download year
  year<- substr(files[i],nchar(files[i])-7,nchar(files[i])-4)
  
  # Set pollutant
  pollutant<- ifelse(substr(files[i],7,11)=="88101", "PM2.5","Ozone")

  
  ## Add additional fields, pad AQS data to fill in missing values and reshape to wide format
  # Read in .csv file
  tempdf<- fread(files[i], sep=",", colClasses = dataCol)

  # Ensure Date.Local is stored as a Date
  tempdf[,`Date Local`:= as.Date(`Date Local`)]
  
  # Remove Canadian monitors and ensure State Code is stored as values
  tempdf<- tempdf[`State Code`!="CC"]
  tempdf[,`State Code`:= as.numeric(`State Code`)]

  # Create the Monitor Name field
  tempdf[,Monitor.Name:= sprintf("%02d-%03d-%04d-%05d-%01d",`State Code`,`County Code`,`Site Num`,`Parameter Code`,POC)]
  
  # Replace NAs in Method Code with "."
  tempdf[,`Method Code`:= as.character(`Method Code`)]
  tempdf[is.na(get("Method Code")), ("Method Code"):="."]
  
  # Generate Monitor Description field
  tempdf[,Monitor.Description:= paste("'",sprintf('MethodCode=%03s',`Method Code`),"','",sprintf('POC=%01d',POC),"'")]
  
  # Remove excess rows and re-order data
  if (pollutant == "PM2.5"){ # For PM, remove 1 HOUR samples and event type != "None" and use Arithmetic Mean value field
    tempdf<- tempdf[`Sample Duration`!="1 HOUR" & `Event Type`=="None" & `Observation Percent`>=75]
    tempdf<- tempdf[,c(15:16,6:7,9,12)]
    
  } else { # For Ozone, remove event type != "None" and use the 1st Max value field
    
    tempdf<- tempdf[`Event Type`=="None" & `Observation Percent`>=75]
    tempdf<- tempdf[,c(15:16,6:7,9,13)]
    tempdf$`1st Max Value`<-tempdf$`1st Max Value`*1000 #convert 1st Max Value field to ppb from ppm
  }

  # Pad data to fill in missing days
  tempdf<- pad(tempdf, interval="day", start_val=as.Date(paste(year,"-01-01",sep="")), end_val=as.Date(paste(year,"-12-31",sep="")), by="Date Local", group=c("Latitude","Longitude","Monitor.Name","Monitor.Description"))
  
  # Rename field names
  names(tempdf)[5]<-"Date.Local"
  names(tempdf)[6]<-"Value"
  
  
  # Reshape data from long to wide format
  tempdf<- dcast(tempdf, Monitor.Name+Monitor.Description+Latitude+Longitude~Date.Local, value.var="Value")

  # Convert data table to data frame
  tempdf<- as.data.frame(tempdf)
  
  # Fill missing values with "."
  tempdf[,-c(1:4)][is.na(tempdf[,-c(1:4)])]<- "."
  
  # Combine values into comma-spearated string
  tempdf$Values<- apply(tempdf[,-c(1:4)], 1, paste, collapse=",")
  
  # Create final BenMAP format
  tempdf<- tempdf[,c(1:4,ncol(tempdf))]
  
  # Assign metric
  if (pollutant == "PM2.5"){
    tempdf$Metric<- "D24HourMean"
  } else {
    tempdf$Metric<- "D8HourMax"
  }
  
  # Assign seasonal metric and statistic
  tempdf$Seasonal.Metric<- NA
  tempdf$Statistic<- NA
  tempdf<- tempdf[,c(1:4,6:8,5)]

  # Update column headers to match BenMAP formatting
  colnames(tempdf)<- c("Monitor Name", "Monitor Description", "Latitude", "Longitude", "Metric", "Seasonal Metric", "Statistic", "Values")
  
  # Write formatted dataset to BenMAP directory
  write.csv(tempdf[,1:8], file= paste("BenMAP Files/EPA Standard Monitors_",pollutant,"_",year,".csv",sep=""),row.names=FALSE, na="")
  
  rm(tempdf)
}

